using System;
using System.Collections.Generic;
using System.Linq;
using AdHocApi;
using AdHocApi.BusinessLayer;
using AdHocApi.Models;
// using System.Windows.Forms;    // Note: remove comment when using with windows forms

/// <summary>
/// These are data-centric code examples for the Categories table.
/// You can cut and paste the respective codes into your application
/// by changing the sample values assigned from these examples.
/// NOTE: This class is sealed and contains private methods because they're
/// not meant to be called by an outside client.  Each method contains
/// code for the respective example being shown.
/// These examples shows how to call the Business Layer (or the Middler Layer or Middler Tier) from a client.
/// A client can be a Windows Form, WPF app, Silverlight app, ASP.NET Web Form/MVC/Razor app, a Web API, a Class, and many more.
/// </summary>
public sealed class CategoryExample
{
    // inject the business layer objects in the constructor
    private readonly ICategoryBusinessLayer _categoryBusinessLayer;

    public CategoryExample(ICategoryBusinessLayer categoryBusinessLayer)
    {
        _categoryBusinessLayer = categoryBusinessLayer;
    }

    /// <summary>
    /// Shows how to Select all records.  It also shows how to sort, bind, and loop through records.
    /// </summary>
    private async void SelectAllAsync()
    {
        // select all records
        List<Category> objCategoriesList = await _categoryBusinessLayer.SelectAllAsync();

        // Example 1:  you can optionally sort the collection in ascending order by your chosen field 
        objCategoriesList.Sort(CategoryBusinessLayer.ByCategoryName);

        // Example 2:  to sort in descending order, add this line to the Sort code in Example 1 
        objCategoriesList.Reverse();

        // Example 3:  directly bind to a GridView - for ASP.NET Web Forms
        // GridView grid = new GridView();
        // grid.DataSource = objCategoriesList;
        // grid.DataBind();

        // Example 4:  loop through all the Categories
        foreach (Category objCategory in objCategoriesList)
        {
            int categoryID = objCategory.CategoryID;
            string categoryName = objCategory.CategoryName;
            string description = objCategory.Description;
        }
    }

    /// <summary>
    /// Shows how to Select all records sorted by column name in either ascending or descending order.
    /// </summary>
    private async void SelectAllWithSortExpression()
    {
        // select all records sorted by CategoryID in ascending order
        string sortBy = "CategoryID"; // ascending order
        //string sortBy = "CategoryID desc"; // descending order

        List<Category> objCategoriesList = await _categoryBusinessLayer.SelectAllAsync(sortBy);
    }

    /// <summary>
    /// Shows how to Select a record by Primary Key.  It also shows how to retrieve Lazily-loaded related Objects.  Related Objects are assigned for each Foreign Key.
    /// </summary>
    private async void SelectByPrimaryKeyAsync()
    {
        int categoryIDSample = 1;

        // select a record by primary key(s)
        Category objCategory = await _categoryBusinessLayer.SelectByPrimaryKeyAsync(categoryIDSample);

        if (objCategory != null)
        {
            // if record is found, a record is returned
            int categoryID = objCategory.CategoryID;
            string categoryName = objCategory.CategoryName;
            string description = objCategory.Description;
        }
    }

    /// <summary>
    /// The example below shows how to Select the CategoryID and CategoryName columns for use with a with a Drop Down List, Combo Box, Checked Box List, List View, List Box, etc
    /// </summary>
    private async void SelectCategoryDropDownListDataAsync()
    {
        List<Category> objCategoriesList = await _categoryBusinessLayer.SelectCategoryDropDownListDataAsync();

        // Example 1:  directly bind to a drop down list - for ASP.NET Web Forms
        // DropDownList ddl1 = new DropDownList();
        // ddl1.DataValueField = "CategoryID";
        // ddl1.DataTextField = "CategoryName";
        // ddl1.DataSource = objCategoriesList;
        // ddl1.DataBind();

        // Example 2:  add each item through a loop - for ASP.NET Web Forms
        // DropDownList ddl2 = new DropDownList();

        // foreach (Category objCategory in objCategoriesList)
        // {
        //     ddl2.Items.Add(new ListItem(objCategory.CategoryName, objCategory.CategoryID.ToString()));
        // }

        // Example 3:  bind to a combo box.  for Windows Forms (WinForms)
        // ComboBox cbx1 = new ComboBox();

        // foreach (Category objCategory in objCategoriesList)
        // {
        //     cbx1.Items.Add(new ListItem(objCategory.CategoryName, objCategory.CategoryID.ToString()));
        // }
    }

    /// <summary>
    /// Shows how to Insert or Create a New Record
    /// </summary>
    private async void Insert()
    {
        // first instantiate a new Category
        Category objCategory = new();

        // assign values you want inserted
        objCategory.CategoryName = "Beverages";
        objCategory.Description = "Soft drinks, coffees, teas, beers, and ales";

        // finally, insert a new record
        // the insert method returns the newly created primary key
        int newlyCreatedPrimaryKey = await _categoryBusinessLayer.InsertAsync(objCategory);
    }

    /// <summary>
    /// Shows how to Update an existing record by Primary Key
    /// </summary>
    private async void UpdateAsync()
    {
        // first instantiate a new Category
        Category objCategory = new();

        // assign the existing primary key(s)
        // of the record you want updated
        objCategory.CategoryID = 1;

        // assign values you want updated
        objCategory.CategoryName = "Beverages";
        objCategory.Description = "Soft drinks, coffees, teas, beers, and ales";

        // finally, update an existing record
        await _categoryBusinessLayer.UpdateAsync(objCategory);
    }

    /// <summary>
    /// Shows how to Delete an existing record by Primary Key
    /// </summary>
    private async void DeleteAsync()
    {
        // delete a record by primary key
        // using business layer;
        await _categoryBusinessLayer.DeleteAsync(9);
    }

    /// <summary>
    /// Shows how to Delete Multiple records by Primary Key
    /// </summary>
    private async void DeleteMultpleAsync()
    {
        // sample data only, in the real world you should use different values
        string ids = "9, 9";

        // split ids into a List
        List<Int32> categoryIDList = ids.Split(",").Select(Int32.Parse).ToList();

        // using business layer, delete multiple records based on a list of ids (primary keys)
        await _categoryBusinessLayer.DeleteMultipleAsync(categoryIDList);
    }

    /// <summary>
    /// Shows how to get the total number of records
    /// </summary>
    private async void GetRecordCountAsync()
    {
        // get the total number of records in the Category table
        int totalRecordCount;

        // using the business layer
        totalRecordCount = await _categoryBusinessLayer.GetRecordCountAsync();
    }

    /// <summary>
    /// Shows how to get the total number of records based on Search Parameters.
    /// </summary>
    private async void GetRecordCountDynamicWhereAsync()
    {
        // search parameters, everything is nullable, only items being searched for should be filled
        // note: fields with String type uses a LIKE search, everything else uses an exact match
        // also, every field you're searching for uses the AND operator
        // e.g. int? productID = 1; string productName = "ch";
        // will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        int? categoryID = null;
        string categoryName = null;

        int totalRecordCount = 0;

        // using business layer
        totalRecordCount = await _categoryBusinessLayer.GetRecordCountDynamicWhereAsync(categoryID, categoryName);
    }

    /// <summary>
    /// Shows how to get a specific number of sorted records, starting from an index.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    /// For example, if there are 200 records take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    /// The example below uses some variables, here are their definitions:
    /// totalRecordCount - total number of records if you were to retrieve everything
    /// startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    /// numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    /// sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    /// sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    /// </summary>
    private async void SelectSkipAndTakeAsync()
    {
        int numberOfRecordsToRetrieve = 10;
        //string sortByDesc = "CategoryID desc";

        // 1. select a specific number of sorted records starting from the index you specify
        List<Category> objCategoriesList;

        // using business layer
        int startRetrievalFromRecordIndex = 0;
        string sortByAsc = "CategoryID";
        objCategoriesList = await _categoryBusinessLayer.SelectSkipAndTakeAsync(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, sortByAsc);

        // to use objCategoriesList please see the SelectAll() method examples
        // No need for Examples 1 and 2 because the Collection here is already sorted

        // Example 2:  directly bind to a GridView - for ASP.NET Web Forms
        // Example 3:  loop through all the Category(s).  The example above will only loop for 10 items.
    }

    /// <summary>
    /// Shows how to get a specific number of sorted records, starting from an index, based on Search Parameters.  The number of records are also retrieved.
    /// </summary>
    private async void SelectSkipAndTakeDynamicWhereAsync()
    {
        int startRetrievalFromRecordIndex = 0;
        int numberOfRecordsToRetrieve = 10;
        string sortBy = "CategoryID";
        //string sortBy = "CategoryID desc";

        List<Category> objCategoriesList;

        // search parameters, everything is nullable, only items being searched for should be filled
        // note: fields with String type uses a LIKE search, everything else uses an exact match
        // also, every field you're searching for uses the AND operator
        // e.g. int? productID = 1; string productName = "ch";
        // will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        int? categoryID = null;
        string categoryName = null;

        // using business layer
        // 1. select a specific number of sorted records starting from the index you specify based on Search Parameters
        objCategoriesList = await _categoryBusinessLayer.SelectSkipAndTakeDynamicWhereAsync(categoryID, categoryName, numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, sortBy);

        // to use objCategoriesList please see the SelectAll() method examples
        // No need for Examples 1 and 2 because the Collection here is already sorted
        // Example 3:  directly bind to a GridView - for ASP.NET Web Forms
        // Example 4:  loop through all the Categories.  The example above will only loop for 10 items.
    }

    /// <summary>
    /// Shows how to get all records based on Search Parameters.
    /// </summary>
    private async void SelectAllDynamicWhereAsync()
    {
        List<Category> objCategoriesList;

        // search parameters, everything is nullable, only items being searched for should be filled
        // note: fields with String type uses a LIKE search, everything else uses an exact match
        // also, every field you're searching for uses the AND operator
        // e.g. int? productID = 1; string productName = "ch";
        // will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        int? categoryID = null;
        string categoryName = null;

        // using business layer
        objCategoriesList = await _categoryBusinessLayer.SelectAllDynamicWhereAsync(categoryID, categoryName);

    }
}
