using System;
using System.Collections.Generic;
using System.Linq;
using AdHocApi;
using AdHocApi.BusinessLayer;
using AdHocApi.Models;
// using System.Windows.Forms;    // Note: remove comment when using with windows forms

/// <summary>
/// These are data-centric code examples for the Suppliers table.
/// You can cut and paste the respective codes into your application
/// by changing the sample values assigned from these examples.
/// NOTE: This class is sealed and contains private methods because they're
/// not meant to be called by an outside client.  Each method contains
/// code for the respective example being shown.
/// These examples shows how to call the Business Layer (or the Middler Layer or Middler Tier) from a client.
/// A client can be a Windows Form, WPF app, Silverlight app, ASP.NET Web Form/MVC/Razor app, a Web API, a Class, and many more.
/// </summary>
public sealed class SupplierExample
{
    // inject the business layer objects in the constructor
    private readonly ISupplierBusinessLayer _supplierBusinessLayer;

    public SupplierExample(ISupplierBusinessLayer supplierBusinessLayer)
    {
        _supplierBusinessLayer = supplierBusinessLayer;
    }

    /// <summary>
    /// Shows how to Select all records.  It also shows how to sort, bind, and loop through records.
    /// </summary>
    private async void SelectAllAsync()
    {
        // select all records
        List<Supplier> objSuppliersList = await _supplierBusinessLayer.SelectAllAsync();

        // Example 1:  you can optionally sort the collection in ascending order by your chosen field 
        objSuppliersList.Sort(SupplierBusinessLayer.ByCompanyName);

        // Example 2:  to sort in descending order, add this line to the Sort code in Example 1 
        objSuppliersList.Reverse();

        // Example 3:  directly bind to a GridView - for ASP.NET Web Forms
        // GridView grid = new GridView();
        // grid.DataSource = objSuppliersList;
        // grid.DataBind();

        // Example 4:  loop through all the Suppliers
        foreach (Supplier objSupplier in objSuppliersList)
        {
            int supplierID = objSupplier.SupplierID;
            string companyName = objSupplier.CompanyName;
            string contactName = objSupplier.ContactName;
            string contactTitle = objSupplier.ContactTitle;
            string address = objSupplier.Address;
            string city = objSupplier.City;
            string region1 = objSupplier.Region1;
            string postalCode = objSupplier.PostalCode;
            string country = objSupplier.Country;
            string phone = objSupplier.Phone;
            string fax = objSupplier.Fax;
            string homePage = objSupplier.HomePage;
        }
    }

    /// <summary>
    /// Shows how to Select all records sorted by column name in either ascending or descending order.
    /// </summary>
    private async void SelectAllWithSortExpression()
    {
        // select all records sorted by SupplierID in ascending order
        string sortBy = "SupplierID"; // ascending order
        //string sortBy = "SupplierID desc"; // descending order

        List<Supplier> objSuppliersList = await _supplierBusinessLayer.SelectAllAsync(sortBy);
    }

    /// <summary>
    /// Shows how to Select a record by Primary Key.  It also shows how to retrieve Lazily-loaded related Objects.  Related Objects are assigned for each Foreign Key.
    /// </summary>
    private async void SelectByPrimaryKeyAsync()
    {
        int supplierIDSample = 1;

        // select a record by primary key(s)
        Supplier objSupplier = await _supplierBusinessLayer.SelectByPrimaryKeyAsync(supplierIDSample);

        if (objSupplier != null)
        {
            // if record is found, a record is returned
            int supplierID = objSupplier.SupplierID;
            string companyName = objSupplier.CompanyName;
            string contactName = objSupplier.ContactName;
            string contactTitle = objSupplier.ContactTitle;
            string address = objSupplier.Address;
            string city = objSupplier.City;
            string region1 = objSupplier.Region1;
            string postalCode = objSupplier.PostalCode;
            string country = objSupplier.Country;
            string phone = objSupplier.Phone;
            string fax = objSupplier.Fax;
            string homePage = objSupplier.HomePage;
        }
    }

    /// <summary>
    /// The example below shows how to Select the SupplierID and CompanyName columns for use with a with a Drop Down List, Combo Box, Checked Box List, List View, List Box, etc
    /// </summary>
    private async void SelectSupplierDropDownListDataAsync()
    {
        List<Supplier> objSuppliersList = await _supplierBusinessLayer.SelectSupplierDropDownListDataAsync();

        // Example 1:  directly bind to a drop down list - for ASP.NET Web Forms
        // DropDownList ddl1 = new DropDownList();
        // ddl1.DataValueField = "SupplierID";
        // ddl1.DataTextField = "CompanyName";
        // ddl1.DataSource = objSuppliersList;
        // ddl1.DataBind();

        // Example 2:  add each item through a loop - for ASP.NET Web Forms
        // DropDownList ddl2 = new DropDownList();

        // foreach (Supplier objSupplier in objSuppliersList)
        // {
        //     ddl2.Items.Add(new ListItem(objSupplier.CompanyName, objSupplier.SupplierID.ToString()));
        // }

        // Example 3:  bind to a combo box.  for Windows Forms (WinForms)
        // ComboBox cbx1 = new ComboBox();

        // foreach (Supplier objSupplier in objSuppliersList)
        // {
        //     cbx1.Items.Add(new ListItem(objSupplier.CompanyName, objSupplier.SupplierID.ToString()));
        // }
    }

    /// <summary>
    /// Shows how to Insert or Create a New Record
    /// </summary>
    private async void Insert()
    {
        // first instantiate a new Supplier
        Supplier objSupplier = new();

        // assign values you want inserted
        objSupplier.CompanyName = "Exotic Liquids";
        objSupplier.ContactName = "Charlotte Cooper";
        objSupplier.ContactTitle = "Purchasing Manager";
        objSupplier.Address = "49 Gilbert St.";
        objSupplier.City = "London";
        objSupplier.Region1 = "abc";
        objSupplier.PostalCode = "EC1 4SD";
        objSupplier.Country = "UK";
        objSupplier.Phone = "(171) 555-2222";
        objSupplier.Fax = "abc";
        objSupplier.HomePage = "abc";

        // finally, insert a new record
        // the insert method returns the newly created primary key
        int newlyCreatedPrimaryKey = await _supplierBusinessLayer.InsertAsync(objSupplier);
    }

    /// <summary>
    /// Shows how to Update an existing record by Primary Key
    /// </summary>
    private async void UpdateAsync()
    {
        // first instantiate a new Supplier
        Supplier objSupplier = new();

        // assign the existing primary key(s)
        // of the record you want updated
        objSupplier.SupplierID = 1;

        // assign values you want updated
        objSupplier.CompanyName = "Exotic Liquids";
        objSupplier.ContactName = "Charlotte Cooper";
        objSupplier.ContactTitle = "Purchasing Manager";
        objSupplier.Address = "49 Gilbert St.";
        objSupplier.City = "London";
        objSupplier.Region1 = "abc";
        objSupplier.PostalCode = "EC1 4SD";
        objSupplier.Country = "UK";
        objSupplier.Phone = "(171) 555-2222";
        objSupplier.Fax = "abc";
        objSupplier.HomePage = "abc";

        // finally, update an existing record
        await _supplierBusinessLayer.UpdateAsync(objSupplier);
    }

    /// <summary>
    /// Shows how to Delete an existing record by Primary Key
    /// </summary>
    private async void DeleteAsync()
    {
        // delete a record by primary key
        // using business layer;
        await _supplierBusinessLayer.DeleteAsync(30);
    }

    /// <summary>
    /// Shows how to Delete Multiple records by Primary Key
    /// </summary>
    private async void DeleteMultpleAsync()
    {
        // sample data only, in the real world you should use different values
        string ids = "30, 30";

        // split ids into a List
        List<Int32> supplierIDList = ids.Split(",").Select(Int32.Parse).ToList();

        // using business layer, delete multiple records based on a list of ids (primary keys)
        await _supplierBusinessLayer.DeleteMultipleAsync(supplierIDList);
    }

    /// <summary>
    /// Shows how to get the total number of records
    /// </summary>
    private async void GetRecordCountAsync()
    {
        // get the total number of records in the Supplier table
        int totalRecordCount;

        // using the business layer
        totalRecordCount = await _supplierBusinessLayer.GetRecordCountAsync();
    }

    /// <summary>
    /// Shows how to get the total number of records based on Search Parameters.
    /// </summary>
    private async void GetRecordCountDynamicWhereAsync()
    {
        // search parameters, everything is nullable, only items being searched for should be filled
        // note: fields with String type uses a LIKE search, everything else uses an exact match
        // also, every field you're searching for uses the AND operator
        // e.g. int? productID = 1; string productName = "ch";
        // will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        int? supplierID = null;
        string companyName = null;
        string contactName = null;
        string contactTitle = null;
        string address = null;
        string city = null;
        string region1 = null;
        string postalCode = null;
        string country = null;
        string phone = null;
        string fax = null;

        int totalRecordCount = 0;

        // using business layer
        totalRecordCount = await _supplierBusinessLayer.GetRecordCountDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);
    }

    /// <summary>
    /// Shows how to get a specific number of sorted records, starting from an index.  The total number of records are also retrieved when using the SelectSkipAndTake() method.
    /// For example, if there are 200 records take only 10 records (numberOfRecordsToRetrieve), starting from the first index (startRetrievalFromRecordIndex = 0)
    /// The example below uses some variables, here are their definitions:
    /// totalRecordCount - total number of records if you were to retrieve everything
    /// startRetrievalFromRecordIndex - the index to start taking records from. Zero (0) E.g. If you want to skip the first 20 records, then assign 19 here.
    /// numberOfRecordsToRetrieve - take n records starting from the startRetrievalFromRecordIndex
    /// sortBy - to sort in Ascending order by Field Name, just assign just the Field Name, do not pass 'asc'
    /// sortBy - to sort in Descending order by Field Name, use the Field Name, a space and the word 'desc'
    /// </summary>
    private async void SelectSkipAndTakeAsync()
    {
        int numberOfRecordsToRetrieve = 10;
        //string sortByDesc = "SupplierID desc";

        // 1. select a specific number of sorted records starting from the index you specify
        List<Supplier> objSuppliersList;

        // using business layer
        int startRetrievalFromRecordIndex = 0;
        string sortByAsc = "SupplierID";
        objSuppliersList = await _supplierBusinessLayer.SelectSkipAndTakeAsync(numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, sortByAsc);

        // to use objSuppliersList please see the SelectAll() method examples
        // No need for Examples 1 and 2 because the Collection here is already sorted

        // Example 2:  directly bind to a GridView - for ASP.NET Web Forms
        // Example 3:  loop through all the Supplier(s).  The example above will only loop for 10 items.
    }

    /// <summary>
    /// Shows how to get a specific number of sorted records, starting from an index, based on Search Parameters.  The number of records are also retrieved.
    /// </summary>
    private async void SelectSkipAndTakeDynamicWhereAsync()
    {
        int startRetrievalFromRecordIndex = 0;
        int numberOfRecordsToRetrieve = 10;
        string sortBy = "SupplierID";
        //string sortBy = "SupplierID desc";

        List<Supplier> objSuppliersList;

        // search parameters, everything is nullable, only items being searched for should be filled
        // note: fields with String type uses a LIKE search, everything else uses an exact match
        // also, every field you're searching for uses the AND operator
        // e.g. int? productID = 1; string productName = "ch";
        // will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        int? supplierID = null;
        string companyName = null;
        string contactName = null;
        string contactTitle = null;
        string address = null;
        string city = null;
        string region1 = null;
        string postalCode = null;
        string country = null;
        string phone = null;
        string fax = null;

        // using business layer
        // 1. select a specific number of sorted records starting from the index you specify based on Search Parameters
        objSuppliersList = await _supplierBusinessLayer.SelectSkipAndTakeDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, numberOfRecordsToRetrieve, startRetrievalFromRecordIndex, sortBy);

        // to use objSuppliersList please see the SelectAll() method examples
        // No need for Examples 1 and 2 because the Collection here is already sorted
        // Example 3:  directly bind to a GridView - for ASP.NET Web Forms
        // Example 4:  loop through all the Suppliers.  The example above will only loop for 10 items.
    }

    /// <summary>
    /// Shows how to get all records based on Search Parameters.
    /// </summary>
    private async void SelectAllDynamicWhereAsync()
    {
        List<Supplier> objSuppliersList;

        // search parameters, everything is nullable, only items being searched for should be filled
        // note: fields with String type uses a LIKE search, everything else uses an exact match
        // also, every field you're searching for uses the AND operator
        // e.g. int? productID = 1; string productName = "ch";
        // will translate to: SELECT....WHERE productID = 1 AND productName LIKE '%ch%'
        int? supplierID = null;
        string companyName = null;
        string contactName = null;
        string contactTitle = null;
        string address = null;
        string city = null;
        string region1 = null;
        string postalCode = null;
        string country = null;
        string phone = null;
        string fax = null;

        // using business layer
        objSuppliersList = await _supplierBusinessLayer.SelectAllDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);

    }
}
